
;**************************************************************************
;*                communication with a PS/2 keyboard                      *
;**************************************************************************
;
;**************************************************************************
;*          v 1.20 - 14.05.2012                                           *
;**************************************************************************
;*          Hardw.: PIC16F687 used                                        *
;*          OSC.......: Internal osc. used                                *
;*          POWER.....:  4x1,2V accu                                      *
;**************************************************************************


;
;keyboard Clock connected to RB4
;keyboard Data  connected to RA4
;

;4 bit communication used
;
; LCD
;	D0 -> n/c
;	D1 -> n/c
;	D2 -> n/c
;	D3 -> n/c
;
;	D4 -> RA0
;	D5 -> RA1
;	D6 -> RA2
;	D7 -> RB7 (RA3 is input only on this processor)
;
;	EN  -> RC0
;	RS  -> RC1
;	R/W -> RC2
;
;HP Display RH6-3068: 2 x 24 characters
;           1313 01S
;           EDMUC161305G


 INCLUDE "P16F687.INC"




;                                      FEDCBA9876543210
_U001			     EQU     B'0000000011000100'	;see Page 197 of Datasheet


;	__CONFIG        _BODEN_OFF & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT
;	__CONFIG        _BODEN_OFF & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _XT_OSC

	__CONFIG	_U001


;==========================================================================
;       Variable Definition
;==========================================================================
;INPUTS
TIMER1		EQU	H'20'		;Used in delay routine
TIMER2		EQU	H'21'		; "	"	"	
data_tmp	EQU	H'22'		;temporary data storage for display communication
data_tmp2	EQU	H'23'		;temporary data storage for display communication
;
kbd_buffer	EQU	H'24'
kbd_bb		EQU	H'25'
c_tmp		EQU	H'26'

fl_pos		EQU	H'27'		;position of last character written to first line

kbd_buffer1	EQU	H'28'
kbd_buffer2	EQU	H'29'
kbd_buffer3	EQU	H'2A'
kbd_buffer4	EQU	H'2B'
kbd_buffer5	EQU	H'2C'
kbd_buffer6	EQU	H'2D'
kbd_buffer7	EQU	H'2E'
kbd_buffer8	EQU	H'2F'
kbd_buffer9	EQU	H'30'

m_esc		EQU	H'31'		;escape key was pressed

kbd_out		EQU	H'32'
kbd_parity	EQU	H'33'

led_dr		EQU	H'34'		;loops for LED disko

led_status	EQU	H'35'

hane		EQU	H'36'		;Hanhofen Editor

kbd_buff_tmp	EQU	H'37'
E0_flag		EQU	H'38'
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.

;		ORG	004
;		goto	int_serv

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		nop
		bcf	STATUS,RP0
		bcf	STATUS,RP1
		clrf	PORTA
		clrf	PORTB
		clrf	PORTC

		bcf	STATUS,RP0	;Switch Back to reg. Bank 2
		bsf	STATUS,RP1	;Switch Back to reg. Bank 2
;
		clrf	ANSEL
		clrf	ANSELH
;
		bcf	STATUS,RP1
		BSF	STATUS,RP0	;Switch to register bank 1

		MOVLW	B'00000000'	;Set PIC options (See datasheet - Pg39).
		MOVWF	OPTION_REG	;Write the OPTION register.

			;bit7 --> PORTA/PORTB Pull-up Enable bit (0 = enabled)
			;bit6 --> Interrupt on rising(1) / falling(0) edge of RA2/INT pin
			;bit5 --> Interrupt on rising edge of RA2/INT pin (0 = Interrupt on rising edge of RA2/INT pin)
			;bit4 --> Interrupt on rising edge of RA2/INT pin (0 = Increment on low-to-high transition on RA2/T0CKI pin)
			;bit3 --> PSA: Prescaler Assignment bit (0 = Prescaler is assigned to the Timer0 module)
			;bit2 - 0 --> PS<2:0>: Prescaler Rate Select bits


		clrf	INTCON		;disable interrupts

		MOVLW	B'01111000'	;RCx = 1 -> input; RCx = 0 -> output
		MOVWF	TRISC
		MOVLW	B'01111111'	;RBx = 1 -> input; RBx = 0 -> output
		MOVWF	TRISB
		MOVLW	B'11110000'	;RAx = 1 -> input; RAx = 0 -> output
		MOVWF	TRISA


		movlw	B'00100000'
		movwf	WPUA		;weak pull-up on PORTA / bit5
		clrf	IOCA		;no interrupt on change for PORTA

		bcf	STATUS,RP0	;Switch Back to reg. Bank 2
		bsf	STATUS,RP1	;Switch Back to reg. Bank 2
;
		movlw	B'00010000'
		movwf	WPUB		;weak pull-up on PORTB / bit4
		movlw	B'00010000'
		movwf	IOCB		;interrupt on change for PORTB / bit4

;Disable Comparator modules
		bcf	CM1CON0,C1ON
		bcf	CM2CON0,C2ON
;

		bcf	STATUS,RP0	;Switch Back to reg. Bank 0
		bcf	STATUS,RP1	;Switch Back to reg. Bank 0

;----------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                        main routine                                     x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		clrf	PORTC

		call	disp_init
		nop
		call	w_welcome

;blink few times with the connected LED (I'm alive)
aaxx		bsf	PORTC,D'007'
		call	dr01
		bcf	PORTC,D'007'
		call	dr01
		bsf	PORTC,D'007'
		call	dr01
		bcf	PORTC,D'007'
		call	dr01

		call	disp_clr2
		call	w_pak
;

wait_f01	btfss	PORTB,D'004'
		goto	cont1
		goto	wait_f01


cont1		call	get_wca

		call	disp_clr
		call	init2

		clrf	fl_pos

		clrf	led_status
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
menu01		call	w_menu
		clrf	m_esc

		clrf	led_status
		call	set_LEDs

menu02		btfsc	PORTB,D'004'
		goto	menu02

		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'05'		;F1 - HEX codes
		btfsc	STATUS,Z
		goto	write_HEX

		movf	kbd_buffer,0
		xorlw	H'06'		;F2 - "Text editor"
		btfsc	STATUS,Z
		goto	han_word


		movf	kbd_buffer,0
		xorlw	H'04'		;F3 - "LED disko"
		btfsc	STATUS,Z
		goto	LED_disko

		goto	menu02

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

han_word	nop
		call	disp_clr
		movlw	B'10000000'
		call	send_comm
		clrf	m_esc
		clrf	hane
		bsf	hane,D'000'		;Hane active

		movlw	D'000'
		movwf	fl_pos

hw_m		btfss	PORTB,D'004'
		goto	get_wca2
		goto	hw_m


get_wca2	bsf	PORTC,D'007'
		call	get_wca_s
		call	set_LEDs
		bcf	PORTC,D'007'

		btfsc	m_esc,D'000'
		goto	menu01
		goto	hw_m

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

LED_disko	nop

		call	disp_clr
		movlw	B'10000000'
		call	send_comm

		clrf	hane

		call	wm_L
		call	wm_E
		call	wm_D
		call	wm_space
		call	wm_d
		call	wm_i
		call	wm_s
		call	wm_k
		call	wm_o
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		movlw	D'005'
		movwf	led_dr

;----- 1. --------------------------
led_loop	movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000111'
		movwf	kbd_out
		call	send_ck
		bsf	PORTC,D'007'
		call	dr01
;----- 2. --------------------------
		movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000000'
		movwf	kbd_out
		call	send_ck
		bcf	PORTC,D'007'
		call	dr01
;----- 3. --------------------------
		movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000010'		;Num Lock
		movwf	kbd_out
		call	send_ck
		bsf	PORTC,D'007'
		call	dr01
;----- 4. --------------------------
		movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000100'		;Caps Lock
		movwf	kbd_out
		call	send_ck
		bcf	PORTC,D'007'
		call	dr01
;----- 5. --------------------------
		movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000001'		;Scroll Lock
		movwf	kbd_out
		call	send_ck
		bsf	PORTC,D'007'
		call	dr01
;----- 6. --------------------------
		movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000100'		;Caps Lock
		movwf	kbd_out
		call	send_ck
		bsf	PORTC,D'007'
		call	dr01
;----- 7. --------------------------
		movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000010'		;Num Lock
		movwf	kbd_out
		call	send_ck
		bcf	PORTC,D'007'
		call	dr01
;----- 8. --------------------------
		movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000000'		;off
		movwf	kbd_out
		call	send_ck
		bcf	PORTC,D'007'
		call	dr01

		decfsz	led_dr,1
		goto	led_loop

		goto	menu01

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
write_HEX	call	disp_clr
		movlw	B'10000000'		;1st line
		call	send_comm
;
		call	wm_C
		call	wm_h
		call	wm_a
		call	wm_r
		call	wm_dvojbodka
		call	wm_space

		clrf	m_esc
		clrf	hane


main		clrf	kbd_buffer1
		clrf	kbd_buffer2
		clrf	kbd_buffer3
		clrf	kbd_buffer4
		clrf	kbd_buffer5
		clrf	kbd_buffer6
		clrf	kbd_buffer7
		clrf	kbd_buffer8
		clrf	kbd_buffer9
;

		btfss	PORTB,D'004'
		goto	get_key_now
		goto	main

get_key_now	nop

		movlw	D'006'
		movwf	fl_pos

		bsf	PORTC,D'007'

		call	get_wca
		call	write_hex_all
		call	set_LEDs

		bcf	PORTC,D'007'

		btfsc	m_esc,D'000'
		goto	menu01
		goto	main

;----------------------------------------------------------------------------
w_menu		call	disp_clr
		movlw	B'10000000'		;1st line
		call	send_comm

		call	wm_M
		call	wm_e
		call	wm_n
		call	wm_u
		call	wm_dvojbodka
		call	wm_space
		call	wm_F
		call	wm_1
		call	wm_minus
		call	wm_H
		call	wm_E
		call	wm_X
		call	wm_space
		call	wm_c
		call	wm_o
		call	wm_d
		call	wm_e
		call	wm_s

		movlw	B'11000000'		;2nd line
		call	send_comm

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_F
		call	wm_2
		call	wm_minus
		call	wm_T
		call	wm_e
		call	wm_x
		call	wm_t
		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_F
		call	wm_3
		call	wm_minus
		call	wm_L
		call	wm_E
		call	wm_D

		return
;----------------------------------------------------------------------------
w_welcome	movlw	B'10000000'		;1st line

		call	wm_P
		call	wm_S
		call	wm_sls
		call	wm_2
		call	wm_space
		call	wm_K
		call	wm_e
		call	wm_y
		call	wm_b
		call	wm_o
		call	wm_a
		call	wm_r
		call	wm_d
		call	wm_space
		call	wm_D
		call	wm_e
		call	wm_m
		call	wm_o

		movlw	B'11000000'		;2nd line
		call	send_comm
		call	wm_P
		call	wm_l
		call	wm_e
		call	wm_a
		call	wm_s
		call	wm_e
		call	wm_space
		call	wm_w
		call	wm_a
		call	wm_i
		call	wm_t
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		return
;-----------------------------------------------------------------------
w_pak		nop				;"Press Any Key"
		movlw	B'11000000'		;2nd line
		call	send_comm

		call	wm_P
		call	wm_r
		call	wm_e
		call	wm_s
		call	wm_s
		call	wm_space
		call	wm_a
		call	wm_n
		call	wm_y
		call	wm_space
		call	wm_k
		call	wm_e
		call	wm_y
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;****************************************************************************
;**                 DISPLAY communication routines                         **
;****************************************************************************
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
wm_space	movlw	' '
		call	send_data
		return

wm_bodka	movlw	'.'
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_carka	movlw	','
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_z1		movlw	'('
		call	send_data
		return

wm_z2		movlw	')'
		call	send_data
		return
;
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_ae		movlw	B'11100001'		;
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_j		movlw	'j'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_oe		movlw	B'11101111'		;
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_q		movlw	'q'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return

wm_z		movlw	'z'
		call	send_data
		return

wm_l_shift	call	wm_L
		call	wm_e
		call	wm_f
		call	wm_t
		call	wm_space
		call	wm_S
		call	wm_H
		call	wm_I
		call	wm_F
		call	wm_T
		return
		
wm_r_shift	call	wm_R
		call	wm_i
		call	wm_g
		call	wm_h
		call	wm_t
		call	wm_space
		call	wm_S
		call	wm_H
		call	wm_I
		call	wm_F
		call	wm_T
		return

wm_caps_lock	call	wm_C
		call	wm_a
		call	wm_p
		call	wm_s
		call	wm_space
		call	wm_L
		call	wm_o
		call	wm_c
		call	wm_k

		movlw	B'00000100'
		xorwf	led_status,1

		return


wm_caps_lock2	nop

		movlw	B'00000100'
		xorwf	led_status,1

		decf	fl_pos,1

		return

wm_al		call	wm_A
		call	wm_r
		call	wm_r
		call	wm_o
		call	wm_w
		call	wm_space
		call	wm_l
		call	wm_e
		call	wm_f
		call	wm_t
		return

wm_ar		call	wm_A
		call	wm_r
		call	wm_r
		call	wm_o
		call	wm_w
		call	wm_space
		call	wm_r
		call	wm_i
		call	wm_g
		call	wm_t
		call	wm_h
		return

wm_ad		call	wm_A
		call	wm_r
		call	wm_r
		call	wm_o
		call	wm_w
		call	wm_space
		call	wm_d
		call	wm_o
		call	wm_w
		call	wm_n
		return

wm_au		call	wm_A
		call	wm_r
		call	wm_r
		call	wm_o
		call	wm_w
		call	wm_space
		call	wm_u
		call	wm_p
		return

wm_pgu		call	wm_P
		call	wm_a
		call	wm_g
		call	wm_e
		call	wm_space
		call	wm_U
		call	wm_p
		return

wm_pgd		call	wm_P
		call	wm_a
		call	wm_g
		call	wm_e
		call	wm_space
		call	wm_D
		call	wm_o
		call	wm_w
		call	wm_n
		return

wm_home		call	wm_H
		call	wm_o
		call	wm_m
		call	wm_e
		return

wm_end		call	wm_E
		call	wm_n
		call	wm_d
		return

wm_insr		call	wm_I
		call	wm_n
		call	wm_s
		call	wm_e
		call	wm_r
		call	wm_t
		return

wm_TAB		call	wm_T
		call	wm_A
		call	wm_B
		return

wm_l_ctrl	call	wm_L
		call	wm_e
		call	wm_f
		call	wm_t
		call	wm_space
		call	wm_C
		call	wm_t
		call	wm_r
		call	wm_l
		return

wm_l_alt	call	wm_L
		call	wm_e
		call	wm_f
		call	wm_t
		call	wm_space
		call	wm_A
		call	wm_l
		call	wm_t
		return

wm_r_ctrl	call	wm_R
		call	wm_i
		call	wm_g
		call	wm_h
		call	wm_t
		call	wm_space
		call	wm_C
		call	wm_t
		call	wm_r
		call	wm_l
		return

wm_r_alt	call	wm_R
		call	wm_i
		call	wm_g
		call	wm_h
		call	wm_t
		call	wm_space
		call	wm_A
		call	wm_l
		call	wm_t
		return

wm_scroll	call	wm_S
		call	wm_c
		call	wm_r
		call	wm_o
		call	wm_l
		call	wm_l
		call	wm_space
		call	wm_L
		call	wm_o
		call	wm_c
		call	wm_k

		movlw	B'00000001'
		xorwf	led_status,1

		return

wm_num_lock	call	wm_N
		call	wm_u
		call	wm_m
		call	wm_space
		call	wm_L
		call	wm_o
		call	wm_c
		call	wm_k

		movlw	B'00000010'
		xorwf	led_status,1

		return

wm_ENTER	call	wm_E
		call	wm_N
		call	wm_T
		call	wm_E
		call	wm_R
		return

wm_del3		call	wm_D
		call	wm_e
		call	wm_l
		return

wm_del		call	wm_B
		call	wm_a
		call	wm_c
		call	wm_k
		return

wm_del2		nop
		decf	fl_pos,1
		movlw	B'10000000'
		iorwf	fl_pos,0		;1st line position
		call	send_comm
		call	wm_space
		decf	fl_pos,1
		return

wm_F1		call	wm_F
		call	wm_1
		return

wm_F2		call	wm_F
		call	wm_2
		return

wm_F3		call	wm_F
		call	wm_3
		return

wm_F4		call	wm_F
		call	wm_4
		return

wm_F5		call	wm_F
		call	wm_5
		return

wm_F6		call	wm_F
		call	wm_6
		return

wm_F7		call	wm_F
		call	wm_7
		return

wm_F8		call	wm_F
		call	wm_8
		return

wm_F9		call	wm_F
		call	wm_9
		return

wm_F10		call	wm_F
		call	wm_1
		call	wm_0
		return

wm_F11		call	wm_F
		call	wm_1
		call	wm_1
		return

wm_F12		call	wm_F
		call	wm_1
		call	wm_2
		return
;-------------------------------------------------------------------------------

;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                      display communication routines                                         x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
disp_init	nop
;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTC
		clrf	PORTA
		call	d2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTC,D'001'
		bcf	PORTC,D'002'
		bcf	PORTC,D'000'	;EN


		movlw	B'00000011'
		movwf	PORTA
		bsf	PORTC,D'000'
		call	dy_comm
		bcf	PORTC,D'000'
		call	d2

		movlw	B'00000011'
		movwf	PORTA
		bsf	PORTC,D'000'
		call	dy_comm
		bcf	PORTC,D'000'
		call	d2

		movlw	B'00000011'
		movwf	PORTA
		bsf	PORTC,D'000'
		call	dy_comm
		bcf	PORTC,D'000'
		call	d2

		movlw	B'00000010'
		movwf	PORTA
		bsf	PORTC,D'000'
		call	dy_comm
		bcf	PORTC,D'000'
		call	d2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return

;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;4-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTC,D'000'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTA			;higher 4 bits are put to RA0-RA3

		bcf	PORTB,D'007'
		movwf	data_tmp2
		btfsc	data_tmp2,D'003'
		bsf	PORTB,D'007'

		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTA			;lower 4 bits are put to RA0-RA3

		bcf	PORTB,D'007'
		movwf	data_tmp2
		btfsc	data_tmp2,D'003'
		bsf	PORTB,D'007'

		call	scbd			;send

		return



;set control bits
scbd		bcf	PORTC,D'002'		;R/W -> 0 (write)
		bsf	PORTC,D'001'		;RS -> 1 (data)
		bsf	PORTC,D'000'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'000'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTC,D'000'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTA			;higher 4 bits are put to RA0-RA3

		bcf	PORTB,D'007'
		movwf	data_tmp2
		btfsc	data_tmp2,D'003'
		bsf	PORTB,D'007'

		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTA			;lower 4 bits are put to RA0-RA3

		bcf	PORTB,D'007'
		movwf	data_tmp2
		btfsc	data_tmp2,D'003'
		bsf	PORTB,D'007'
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTC,D'002'		;R/W -> 0 (write)
		bcf	PORTC,D'001'		;RS -> 0 (command)
		bsf	PORTC,D'000'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'000'		;EN -> 0
		call	d1			;wait a little bit...
		return

;----------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

		call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;----------------------------------------------------------------------------
disp_clr	call	disp_clr1
		call	disp_clr2
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP21	MOVLW   D'255'
		MOVWF   TIMER1
DEL_LOOP22	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP22
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP21
		return

d1		call	d1b
		call	d1b
		call	d1b
		return


d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return


d2sx		MOVLW   D'010'
		MOVWF   TIMER2
DEL_LOOP21sx	MOVLW   D'255'
		MOVWF   TIMER1
DEL_LOOP22sx	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP22sx
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP21sx
		return
;----------------------------------------------

dy_comm		call	dy_comm2
		call	dy_comm2
		call	dy_comm2
		call	dy_comm2


		return


dy_comm2	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return

;----------------------------------------------
d57		MOVLW   D'030'
		MOVWF   TIMER2
DEL_LOOP5721	MOVLW   D'255'
		MOVWF   TIMER1
DEL_LOOP5722	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP5722
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP5721
		return

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;****************************************************************************
;****************************************************************************
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;		*********************************************
;		*               delay routine               *
;		*********************************************

;----------------------------------------------------------------------------
dr03		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP3_1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1
		RETURN
;-------------------------------------
dr01		call	dr03
		return
;----------------------------------------------------------------------------
dr01s		MOVLW   D'010'
		MOVWF   TIMER2
DEL_LOOP3_1s	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2s	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2s
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1s
		RETURN

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;*****************************************************************************
;***   KEYBAORD COMMUNICATION AND SERVICE ROUTINES                       *****
;*****************************************************************************
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


;get key-code - simple
get_wca_s	call	get_datafk
		movf	kbd_buffer,0
		movwf	kbd_buffer1

		clrf	E0_flag
		call	write_char2
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

;get key-code from the keyboard and write received char to display
get_wca		clrf	E0_flag
		call	get_datafk
		movf	kbd_buffer,0
		movwf	kbd_buffer1		;store the value to kbd_bufferx
;it is Ex?
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		goto	prefix_Ex

;simple scan code - next expected scan code is F0.....

		bsf	E0_flag,D'000'		;for the routine write_hex_all
w201		btfsc	PORTB,D'004'
		goto	w201

		call	get_datafk
		movf	kbd_buffer,0
		movwf	kbd_buffer2

w202		btfsc	PORTB,D'004'
		goto	w202

		call	get_datafk
		movf	kbd_buffer,0
		movwf	kbd_buffer3

		call	write_char
		return
;-----------------------------------------------------------------
;now we got E0 or E1
prefix_Ex	nop

w100		btfsc	PORTB,D'004'
		goto	w100

		call	get_datafk
		movf	kbd_buffer,0
		movwf	kbd_buffer2

w102		btfsc	PORTB,D'004'
		goto	w102

		call	get_datafk
		movf	kbd_buffer,0
		movwf	kbd_buffer3

w103		btfsc	PORTB,D'004'
		goto	w103

		call	get_datafk
		movf	kbd_buffer,0
		movwf	kbd_buffer4


w104		btfsc	PORTB,D'004'
		goto	w104

		call	get_datafk
		movf	kbd_buffer,0
		movwf	kbd_buffer5

		call	write_char
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;get a transmission from the keyboard
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

get_datafk	btfsc	PORTB,D'004'
		goto	kbd_err01		;clock level is 1 isntead of expected 0
		btfsc	PORTA,D'004'
		goto	kbd_err02		;data level is 1 istead of expected 0 (start bit)
;
		clrf	kbd_buffer

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		bsf	kbd_buffer,D'000'

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		bsf	kbd_buffer,D'001'

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		bsf	kbd_buffer,D'002'

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		bsf	kbd_buffer,D'003'

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		bsf	kbd_buffer,D'004'

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		bsf	kbd_buffer,D'005'

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		bsf	kbd_buffer,D'006'

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		bsf	kbd_buffer,D'007'

;get parity bit

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		nop

;compare parity bit

		nop				;not now.... :-)

;wait for communication end - last bit should be "1"

		call	get_bitfk
		btfsc	kbd_bb,D'004'
		nop

		nop				;no check right now...

		return				;keyboard code is in kbd_buffer
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;write HEX code of the character
write_hex_all	call	disp_clr2

		movlw	B'11000000'		;2nd line
		call	send_comm

		call	wm_H
		call	wm_E
		call	wm_X
		call	wm_space
		call	wm_dvojbodka
		call	wm_space
;
		movf	kbd_buffer1,0
		movwf	kbd_buffer
		call	write_hex

		movf	kbd_buffer2,0
		xorlw	D'000'
		btfsc	STATUS,Z
		return

		btfss	E0_flag,D'000'
		goto	x_minus
		call	wm_hvizdicka
		goto	x_cnt01

x_minus		call	wm_minus

x_cnt01		movf	kbd_buffer2,0
		movwf	kbd_buffer
		call	write_hex

		movf	kbd_buffer3,0
		xorlw	D'000'
		btfsc	STATUS,Z
		return

		btfsc	E0_flag,D'000'
		goto	x_minus2
		call	wm_hvizdicka
		goto	x_cnt02

x_minus2	call	wm_minus

x_cnt02		movf	kbd_buffer3,0
		movwf	kbd_buffer
		call	write_hex

		movf	kbd_buffer4,0
		xorlw	D'000'
		btfsc	STATUS,Z
		return

		call	wm_minus

		movf	kbd_buffer4,0
		movwf	kbd_buffer
		call	write_hex

		movf	kbd_buffer5,0
		xorlw	D'000'
		btfsc	STATUS,Z
		return

		call	wm_minus

		movf	kbd_buffer5,0
		movwf	kbd_buffer
		call	write_hex

		movf	kbd_buffer6,0
		xorlw	D'000'
		btfsc	STATUS,Z
		return

		call	wm_minus

		movf	kbd_buffer6,0
		movwf	kbd_buffer
		call	write_hex

		movf	kbd_buffer7,0
		xorlw	D'000'
		btfsc	STATUS,Z
		return

		call	wm_minus

		movf	kbd_buffer7,0
		movwf	kbd_buffer
		call	write_hex

		movf	kbd_buffer8,0
		xorlw	D'000'
		btfsc	STATUS,Z
		return

		call	wm_minus

		movf	kbd_buffer8,0
		movwf	kbd_buffer
		call	write_hex

		movf	kbd_buffer9,0
		xorlw	D'000'
		btfsc	STATUS,Z
		return

		call	wm_minus

		movf	kbd_buffer9,0
		movwf	kbd_buffer
		call	write_hex


		return
;--------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
write_char	bcf	E0_flag,D'001'
		movlw	B'10000110'
		call	send_comm
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		

write_char2	movlw	B'10000000'
		iorwf	fl_pos,0		;1st line position
		call	send_comm

		movf	kbd_buffer1,0

		btfsc	hane,D'000'
		goto	gghh12

		xorlw	H'E0'
		btfss	STATUS,Z
		goto	gghh11
;E0 - we have to concider 2nd code
		movf	kbd_buffer2,0
		bsf	E0_flag,D'001'
		goto	gghh12

gghh11		movf	kbd_buffer1,0
gghh12		call	write_ascii_c

		incf	fl_pos,1
		movf	fl_pos,0
		xorlw	D'24'
		btfsc	STATUS,Z
		goto	next_line

		movf	fl_pos,0
		xorlw	D'88'
		btfsc	STATUS,Z
		goto	clear_txt

		call	d2sx

		return

next_line	clrf	fl_pos
		bsf	fl_pos,D'006'

		call	d2sx
		return

clear_txt	clrf	fl_pos
		call	d2sx

		call	d2sx
		return

;-----------------------------------------------------------------------------
;get bit from the keyboard during transmission
get_bitfk	nop
kbd_l2		btfss	PORTB,D'004'
		goto	kbd_l2			;wait to 1 on clock line
kbd_l3		btfsc	PORTB,D'004'
		goto	kbd_l3			;and now wait to 0 on clock
;
		movf	PORTA,0
		andlw	B'00010000'		;now we have the bit in W7
		movwf	kbd_bb

;and now wait until 1 is back...
wuoib		nop
		btfss	PORTB,D'004'
		goto	wuoib

		return
;-----------------------------------------------------------------------------
kbd_err01	nop
		return
;-----------------------------
kbd_err02	nop
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
write_ascii_c	movwf	c_tmp			;CAPITAL letters...

		btfss	E0_flag,D'001'
		goto	wac_nr1

;special characters - scan codes E0-xx
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_al

		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_ar

		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_au

		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_ad

		movf	c_tmp,0
		xorlw	H'5A'
		btfsc	STATUS,Z
		goto	wm_ENTER

		movf	c_tmp,0
		xorlw	H'4A'
		btfsc	STATUS,Z
		goto	wm_sls

		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_pgu

		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_pgd

		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_home

		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_end

		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_del3

		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_insr

		movf	c_tmp,0
		xorlw	H'11'
		btfsc	STATUS,Z
		goto	wm_r_alt

		movf	c_tmp,0
		xorlw	H'14'
		btfsc	STATUS,Z
		goto	wm_r_ctrl

		goto	wm_minus

		return
;------------------------------------------------
wac_nr1		btfsc	led_status,D'002'
		goto	cap_letters
; -----------------------------------------------
;small letters
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_w
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_e
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_r
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_t
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_u
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_i
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_o
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_p
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_a
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_s
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_d
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_f
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_g
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_h
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_j
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_k
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_l
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_x
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_c
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_v
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_b
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_n
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_m


		goto	general_sgn
; -----------------------------------------------
cap_letters	xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;

general_sgn	nop
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka
;               ---------------------
		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
		goto	ww_escape
;               ---------------------

		movf	c_tmp,0
		xorlw	H'41'
		btfsc	STATUS,Z
		goto	wm_carka
;
		movf	c_tmp,0
		xorlw	H'49'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'4A'
		btfsc	STATUS,Z
		goto	wm_minus
;
		btfss	hane,D'000'
		goto	cont95
;special part for HANE -----------------

		movf	c_tmp,0
		xorlw	H'58'
		btfsc	STATUS,Z
		goto	wm_caps_lock2

		movf	c_tmp,0
		xorlw	H'66'
		btfsc	STATUS,Z
		goto	wm_del2

		decf	fl_pos,1
		
		return


;special part for HEX ------------------
cont95		movf	c_tmp,0
		xorlw	H'59'
		btfsc	STATUS,Z
		goto	wm_r_shift

		movf	c_tmp,0
		xorlw	H'12'
		btfsc	STATUS,Z
		goto	wm_l_shift
		
		movf	c_tmp,0
		xorlw	H'58'
		btfsc	STATUS,Z
		goto	wm_caps_lock

		movf	c_tmp,0
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_TAB

		movf	c_tmp,0
		xorlw	H'14'
		btfsc	STATUS,Z
		goto	wm_l_ctrl

		movf	c_tmp,0
		xorlw	H'11'
		btfsc	STATUS,Z
		goto	wm_l_alt

		movf	c_tmp,0
		xorlw	H'5A'
		btfsc	STATUS,Z
		goto	wm_ENTER

		movf	c_tmp,0
		xorlw	H'77'
		btfsc	STATUS,Z
		goto	wm_num_lock

		movf	c_tmp,0
		xorlw	H'7E'
		btfsc	STATUS,Z
		goto	wm_scroll

		movf	c_tmp,0
		xorlw	H'66'
		btfsc	STATUS,Z
		goto	wm_del

		movf	c_tmp,0
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_F1

		movf	c_tmp,0
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_F2

		movf	c_tmp,0
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_F3

		movf	c_tmp,0
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_F4

		movf	c_tmp,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_F5

		movf	c_tmp,0
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_F6

		movf	c_tmp,0
		xorlw	H'83'
		btfsc	STATUS,Z
		goto	wm_F7

		movf	c_tmp,0
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_F8

		movf	c_tmp,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_F9

		movf	c_tmp,0
		xorlw	H'9'
		btfsc	STATUS,Z
		goto	wm_F10

		movf	c_tmp,0
		xorlw	H'78'
		btfsc	STATUS,Z
		goto	wm_F11

		movf	c_tmp,0
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_F12

;not known character - either transmission error or not decoded... - do nothing...

		decf	fl_pos,0
		
		return
;---------------------------------------------------------
ww_escape	bsf	m_esc,D'000'
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; send data to the keyboard
;-----------------------------------------------------------------------------

send_ck		nop
		clrf	kbd_parity
		bsf	kbd_parity,D'000'

;-----------------------------------------------------------
;1.1. set RB4 (keaboard clock) to output ----------------------
		bsf	STATUS,RP0
		MOVLW	B'01101111'	;RBx = 1 -> input; RBx = 0 -> output
		MOVWF	TRISB
		bcf	STATUS,RP0
;-----------------------------------------------------------
;1.2. put KBD CLK to 0 ----------------------------------------
		bcf	PORTB,D'004'
		call	d57		;wait a while
;-----------------------------------------------------------
;2. put KBD DATA to 0 ---------------------------------------
		bsf	STATUS,RP0
		MOVLW	B'11100000'	;RAx = 1 -> input; RAx = 0 -> output
		MOVWF	TRISA
		bcf	STATUS,RP0

		bcf	PORTA,D'004'
;-----------------------------------------------------------
;3. release KBD CLK -----------------------------------------
		bsf	PORTB,D'004'

		bsf	STATUS,RP0
		MOVLW	B'01111111'	;RBx = 1 -> input; RBx = 0 -> output
		MOVWF	TRISB
		bcf	STATUS,RP0
;-----------------------------------------------------------
;4. wait for keayboard to bring the kbd clk low ------------
sck01		btfsc	PORTB,D'004'
		goto	sck01


		movf	kbd_out,0
		andlw	B'00000001'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000010'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000100'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00001000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00010000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00100000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'01000000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'10000000'
		call	send_kbd_bit

;now send the parity bit...
		movf	kbd_parity,0
		andlw	B'00000001'
		call	send_kbd_bit

;		movlw	B'00000001'		;stop bit
;		andlw	B'00000001'
;		call	send_kbd_bit

		bsf	STATUS,RP0
		MOVLW	B'11110000'	;RAx = 1 -> input; RAx = 0 -> output
		MOVWF	TRISA
		bcf	STATUS,RP0

w0079		btfsc	PORTB,D'004'
		goto	w0079

w0080		btfss	PORTB,D'004'
		goto	w0080

		return

;-----------------------------------------------------------
send_kbd_bit	btfsc	STATUS,Z
		goto	send_kbd0
		goto	send_kbd1

send_kbd0	bcf	PORTA,D'004'

w0077		btfss	PORTB,D'004'
		goto	w0077

w0078		btfsc	PORTB,D'004'
		goto	w0078

		return

send_kbd1	bsf	PORTA,D'004'
		movlw	B'00000001'
		xorwf	kbd_parity,1
		goto	w0077
;-----------------------------------------------------------------------------
;set keyboard LEDs based on value in led_status
;
set_LEDs	movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit

		movf	led_status,0
		movwf	kbd_out
		call	send_ck

		call	dr01s

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

		END